Package: msPurity
Type: Package
Title: Automated Evaluation of Precursor Ion Purity for Mass
        Spectrometry Based Fragmentation in Metabolomics
Version: 1.36.0
Date: 2024-05-09
Authors@R: c(
    person(given = "Thomas N.", family = "Lawson",
        email = "thomas.nigel.lawson@gmail.com", role = c("aut", "cre"),
        comment = c(ORCID = "0000-0002-5915-7980")),
    person(given = "Ralf", family = "Weber", role = "ctb"),
    person(given = "Martin", family = "Jones", role = "ctb"),
    person(given = "Julien", family = "Saint-Vanne", role = "ctb"),
    person(given = "Andris", family = "Jankevics", role = "ctb"),
    person(given = "Mark", family = "Viant", role = "ths"),
    person(given = "Warwick", family = "Dunn", role = "ths"))
URL: https://github.com/computational-metabolomics/msPurity/
Description: msPurity R package was developed to: 
     1) Assess the spectral quality of fragmentation spectra by evaluating the "precursor ion purity". 
     2) Process fragmentation spectra. 
     3) Perform spectral matching.
    What is precursor ion purity? -What we call "Precursor ion purity" is a measure of the 
    contribution of a selected precursor peak in an isolation window used for fragmentation. 
    The simple calculation involves dividing the intensity of the selected precursor peak by the 
    total intensity of the isolation window. When assessing MS/MS spectra this calculation is 
    done before and after the MS/MS scan of interest and the purity is interpolated at the 
    recorded time of the MS/MS acquisition. Additionally, isotopic peaks can be removed, 
    low abundance peaks are removed that are thought to have limited contribution to the 
    resulting MS/MS spectra and the isolation efficiency of the mass spectrometer can be used 
    to normalise the intensities used for the calculation.
Encoding: UTF-8
License: GPL-3 + file LICENSE
BugReports: https://github.com/computational-metabolomics/msPurity/issues/new
Depends: Rcpp
Imports: plyr, dplyr, dbplyr, magrittr, foreach, parallel, doSNOW,
        stringr, mzR, reshape2, fastcluster, ggplot2, DBI, RSQLite
Suggests: MSnbase, testthat, xcms, BiocStyle, knitr, rmarkdown,
        msPurityData, CAMERA, RPostgres, RMySQL
VignetteBuilder: knitr
RoxygenNote: 7.3.1
Roxygen: list(markdown = TRUE)
biocViews: MassSpectrometry, Metabolomics, Software
Collate: 'all-generics.R' 'averaging.R' 'combineAnnotations.R'
        'create-database.R' 'createDatabase.R' 'flag-filter-remove.R'
        'iw-norm.R' 'matching-algs.R' 'meta_extract.R' 'msPurity.R'
        'pcalc.R' 'purityA-0-class.R' 'purityA-av-spectra.R'
        'purityA-constructor.R' 'purityA-create-msp.R'
        'purityA-filter-frag-spectra.R' 'purityA-frag4feature.R'
        'purityA-validate.R' 'purityD-class.R' 'purityD-constructor.R'
        'purityD-av-spectra.R' 'purityD-dims-purity.R'
        'purityD-fileList.R' 'purityD-filterp.R' 'purityD-subtract.R'
        'purityD-writeOut.R' 'purityX-class.R' 'purityX-constructor.R'
        'spectral-matching.R' 'spectralMatching.R' 'splinepurity.R'
git_url: https://git.bioconductor.org/packages/msPurity
git_branch: RELEASE_3_22
git_last_commit: 98f4ee5
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:25:01 UTC; biocbuild
Author: Thomas N. Lawson [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5915-7980>),
  Ralf Weber [ctb],
  Martin Jones [ctb],
  Julien Saint-Vanne [ctb],
  Andris Jankevics [ctb],
  Mark Viant [ths],
  Warwick Dunn [ths]
Maintainer: Thomas N. Lawson <thomas.nigel.lawson@gmail.com>
