# Generated by roxygen2: do not edit by hand

export(Getfiles)
export(assessPuritySingle)
export(averageSpectraSingle)
export(combineAnnotations)
export(createDatabase)
export(create_database)
export(dimsPredictPuritySingle)
export(flag_remove)
export(get_additional_mzml_meta)
export(groupPeaksEx)
export(iwNormGauss)
export(iwNormQE.5)
export(iwNormRcosine)
export(pcalc)
export(purityA)
export(purityD)
export(purityX)
export(spectralMatching)
export(spectral_matching)
export(subtractMZ)
exportMethods(averageAllFragSpectra)
exportMethods(averageInterFragSpectra)
exportMethods(averageIntraFragSpectra)
exportMethods(averageSpectra)
exportMethods(createMSP)
exportMethods(dimsPredictPurity)
exportMethods(filterFragSpectra)
exportMethods(filterp)
exportMethods(frag4feature)
exportMethods(getP)
exportMethods(groupPeaks)
exportMethods(show)
exportMethods(subtract)
exportMethods(validate)
exportMethods(writeOut)
import(dbplyr)
import(doSNOW)
import(fastcluster)
import(foreach)
import(mzR)
import(parallel)
import(plyr)
import(reshape2)
import(stringr)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(magrittr,"%>%")
importFrom(stats,approxfun)
importFrom(stats,dnorm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,sd)
importFrom(utils,fix)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,write.csv)
