---
title: "msPurity spectral database schema"
author: "Thomas N. Lawson"
date: "`r Sys.Date()`"
output: 
  BiocStyle::html_document:
    toc: true
    toc_depth: 3  
    number_sections: true  
    toc_float: true
    
bibliography: mspurity.bib
vignette: >
  %\VignetteIndexEntry{msPurity spectral database schema}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


# Spectral database schema

```{r, echo=FALSE}
htmltools::includeHTML("spectral-database-schema.html")
```

